package sdlip;

/**
 * This interface defines tag names of the DAV namespace "DAV:".
 * Refer to the <a href='http://www.webdav.org/dasl/'>DAV specification</a> for more information.
 */

public interface DAV {

  /** DAV namespace */
  final String Namespace = "DAV:";

  // elements usable in the DAV:where and DAV:orderby clause
  final String where = Namespace + "where";

  /** The <tt>DAV:and</tt> operator performs a logical AND operation on the expressions it contains. */
  final String and = Namespace + "and";
  /** The <tt>DAV:or</tt> operator performs a logical OR operation on the values it contains. */
  final String or = Namespace + "or";
  /** The <tt>DAV:not</tt> operator performs a logical NOT operation on the values it contains. */
  final String not = Namespace + "not";

  /** true */
  final String TRUE = Namespace + "true";
  /** false */
  final String FALSE = Namespace + "false";

  /** The <tt>DAV:lt</tt>, <tt>DAV:lte</tt>, <tt>DAV:gt</tt>, and <tt>DAV:gte</tt>
   * operators provide comparisons on property values,
   * using less-than, less-than or equal,
   * greater-than, and greater-than or equal respectively.
   * The <tt>DAV:casesensitive</tt> attribute may be used with these elements.
   */
  final String lt = Namespace + "lt";
  final String lte = Namespace + "lte";
  final String gt = Namespace + "gt";
  final String gte = Namespace + "gte";

  /** The <tt>DAV:eq</tt> operator provides simple equality matching on property values. 
   * The <tt>DAV:casesensitive</tt> attribute may be used with this element.
   */
  final String eq = Namespace + "eq";

  /** The optional <tt>DAV:isdefined</tt> operator allows clients to determine whether a property is defined on a resource. */
  final String isdefined = Namespace + "isdefined";

  /** The <tt>DAV:like</tt> is an optional operator intended to give simple wildcard-based pattern matching ability to clients. 
   * The operator takes two arguments. 
   * The first argument is a <tt>DAV:prop</tt> element identifying a single property to evaluate. 
   * The second argument is a <tt>DAV:literal</tt> element that gives the pattern matching string.*/
  final String like = Namespace + "like";

  final String order = Namespace + "order";
  final String ascending = Namespace + "ascending";
  final String descending = Namespace + "descending";

  /** The <tt>DAV:contains</tt> operator is an optional operator that provides content-based search capability.
   * This operator implicitly searches against
   * the text content of a resource, not against content of properties.
   * The <tt>DAV:contains</tt> operator is intentionally not overly constrained, in order
   * to allow the server to do the best job it can in performing the search. */
  final String contains = Namespace + "contains";

  /** <tt>DAV:literal</tt> allows literal values to be placed in an expression. */
  final String literal = Namespace + "literal";

  /** The DAV:orderby element specifies the ordering of the result set.
   * It contains one or more DAV:order elements, each of which specifies a
   * comparison between two items in the result set.
   * Informally, a comparison specifies a test that determines whether one resource appears
   * before another in the result set. Comparisons are applied in the order they
   * occur in the <tt>DAV:orderby</tt> element, earlier comparisons being
   * more significant. */
  final String orderby = Namespace + "orderby";

  /** Property type that specifies content length */
  final String getcontentlength = Namespace + "getcontentlength";

  /** Property type that specifies content type */
  final String getcontenttype = Namespace + "getcontenttype";

  /** The DAV:score XML element is a synthetic property whose value is defined
   * only in the context of a query result where the server computes
   * a score, e.g. based on relevance. It may be used in <tt>DAV:select</tt> or
   * <tt>DAV:orderby</tt> elements. Servers SHOULD support this property. The
   * value is a string representing the score, an integer from zero to 10000 inclusive,
   * where a higher value indicates a higher score (e.g. more relevant). */
  final String score = Namespace + "score";

  // attributes

  /** The <tt>DAV:casesensitive</tt> attribute allows clients to specify case-sensitive
   * or case-insensitive behavior for <tt>DAV:basicsearch</tt> operators. 
   * The possible values for <tt>DAV:casesensitive</tt> are "1" or "0".
   * The "1" value indicates case-sensitivity. The "0" value indicates
   * case-insensitivity. The default value is server-specified. */
  final String casesensitive = Namespace + "casesensitive";

  // internal stuff

  /** Constants used in the SDLIP/DASL binding */

  final int eBadRequest = 400;
  final int eUnprocessableEntity = 422;
  final int eInsufficientStorage = 507;
  final int rMultiStatus = 207;

  /** Elements used in the SDLIP/DASL binding */

  final String searchrequest = Namespace + "searchrequest";
  final String prop = Namespace + "prop";
  final String response = Namespace + "response";
  final String href = Namespace + "href";
  final String propstat = Namespace + "propstat";
  final String limit = Namespace + "limit";
  final String nresults = Namespace + "nresults";
  final String scope = Namespace + "scope";
  final String multistatus = Namespace + "multistatus";
  final String responsedescription = Namespace + "responsedescription";
  final String redirectarbiter = Namespace + "redirectarbiter";
  final String status = Namespace + "status";
  final String basicsearch = Namespace + "basicsearch";
  final String allprop = Namespace + "allprop";
  final String select = Namespace + "select";
  final String from = Namespace + "from";
  final String depth = Namespace + "depth";


}
