package sdlip;

/**
 * This interface defines some tag names of the DublinCore namespace "http://purl.org/metadata/dublin_core#".
 * Refer to the <a href='http://purl.org/dc/'>DublinCore homepage</a> for more information.
 */

public interface DublinCore {

  /** Namespace URI of this schema */
  public static final String Namespace = "http://purl.org/metadata/dublin_core#";

  /** A rights management statement, an identifier that
  links to a rights management statement, or an identifier that links
  to a service providing information about rights management for the
  resource. */
  public static final String Rights = Namespace + "Rights";

  /** The data format of the resource, used to identify the
  software and possibly hardware that might be needed to display or
  operate the resource. For the sake of interoperability, Format
  should be selected from an enumerated list that is currently under
  development in the workshop series. */
  public static final String Format = Namespace + "Format";

  /** Information about a second resource from which the
  present resource is derived. While it is generally recommended that
  elements contain information about the present resource only, this
  element may contain a date, creator, format, identifier, or other
  metadata for the second resource when it is considered important for
  discovery of the present resource; recommended best practice is to
  use the Relation element instead.  For example, it is possible to
  use a Source date of 1603 in a description of a 1996 film adaptation
  of a Shakespearean play, but it is preferred instead to use Relation
  "IsBasedOn" with a reference to a separate resource whose
  description contains a Date of 1603. Source is not applicable if the
  present resource is in its original form. */
  public static final String Source = Namespace + "Source";

  /** The entity responsible for making the resource
  available in its present form, such as a publishing house, a
  university department, or a corporate entity. */
  public static final String Publisher = Namespace + "Publisher";

  /** An identifier of a second resource and its
  relationship to the present resource. This element permits links
  between related resources and resource descriptions to be
  indicated. Examples include an edition of a work (IsVersionOf), a
  translation of a work (IsBasedOn), a chapter of a book (IsPartOf),
  and a mechanical transformation of a dataset into an image
  (IsFormatOf). For the sake of interoperability, relationships should
  be selected from an enumerated list that is currently under
  development in the workshop series. */
  public static final String Relation = Namespace + "Relation";

  /** The spatial or temporal characteristics of the
  intellectual content of the resource. Spatial coverage refers to a
  physical region (e.g., celestial sector); use coordinates (e.g.,
  longitude and latitude) or place names that are from a controlled
  list or are fully spelled out. Temporal coverage refers to what the
  resource is about rather than when it was created or made available
  (the latter belonging in the Date element); use the same date/time
  format (often a range) [Date and Time Formats (based on ISO8601),
  W3C Technical Note, http://www.w3.org/TR/NOTE-datetime] as
  recommended for the Date element or time periods that are from a
  controlled list or are fully spelled out. */
  public static final String Coverage = Namespace + "Coverage";

  /** The language of the intellectual content of the
  resource. Where practical, the content of this field should coincide
  with RFC 1766 [Tags for the Identification of Languages,
  http://ds.internic.net/rfc/rfc1766.txt ]; examples include en, de,
  es, fi, fr, ja, th, and zh. */
  public static final String Language = Namespace + "Language";

  /** A string or number used to uniquely identify the
  resource. Examples for networked resources include URLs and URNs
  (when implemented). Other globally-unique identifiers, such as
  International Standard Book Numbers (ISBN) or other formal names are
  also candidates for this element. */
  public static final String Identifier = Namespace + "Identifier";

  /** The name given to the resource, usually by the Creator
  or Publisher. */
  public static final String Title = Namespace + "Title";

  /** A person or organization not specified in a Creator
  element who has made significant intellectual contributions to the
  resource but whose contribution is secondary to any person or
  organization specified in a Creator element (for example, editor,
  transcriber, and illustrator). */
  public static final String Contributor = Namespace + "Contributor";

  /**  A textual description of the content of the resource,
  including abstracts in the case of document-like objects or content
  descriptions in the case of visual resources. */
  public static final String Description = Namespace + "Description";

  /** The person or organization primarily responsible for
  creating the intellectual content of the resource. For example,
  authors in the case of written documents, artists, photographers, or
  illustrators in the case of visual resources. */
  public static final String Creator = Namespace + "Creator";

  /** The topic of the resource. Typically, subject will be
  expressed as keywords or phrases that describe the subject or
  content of the resource. The use of controlled vocabularies and
  formal classification schemes is encouraged. */
  public static final String Subject = Namespace + "Subject";

  /** A date associated with the creation or availability of
  the resource. Such a date is not to be confused with one belonging
  in the Coverage element, which would be associated with the resource
  only insofar as the intellectual content is somehow about that
  date. Recommended best practice is defined in a profile of ISO 8601
  [Date and Time Formats (based on ISO8601), W3C Technical Note,
  http://www.w3.org/TR/NOTE-datetime] that includes (among others)
  dates of the forms YYYY and YYYY-MM-DD. In this scheme, for example,
  the date 1994-11-05 corresponds to November 5, 1994. */
  public static final String Date = Namespace + "Date";

  /** The category of the resource, such as home page,
  novel, poem, working paper, technical report, essay, dictionary. For
  the sake of interoperability, Type should be selected from an
  enumerated list that is currently under development in the workshop
  series. */
  public static final String Type = Namespace + "Type";

}
