package sdlip.examples;
//
// Encyclopedia Britannica Library Service Proxy
//
// Author: Arturo Crespo
// Creation Date: May 20, 2000
//
//

//package sdlip.examples;

import sdlip.SDLIP;
import sdlip.DAV;

import sdlip.ResultAccess;
import sdlip.Search;
import sdlip.Metadata;
import sdlip.SDLIPException;
/* transport module libraries */
import sdlip.helpers.*;
/* DOM-enabled XMLObject */
import sdlip.xml.dom.*;
/* holder for integer values */
import org.omg.CORBA.IntHolder;

import org.w3c.dom.*;

import java.util.Hashtable;


public class BritannicaLSP implements Search, ResultAccess, Metadata, Timeoutable {

  class Session {

    int SID;
    Timer timer;
    Britannica service;

    public Session(int SID, String query, int timeout, BritannicaLSP lsp) {

      this.SID = SID;
      this.timer = timeout == 0 ? null : new Timer(timeout, lsp, this);
      service = new Britannica(query);
    }

    public Element getDocs(String docSelection, sdlip.XMLObject docProps) throws SDLIPException {

      Element eSearchResult = XMLObject.getDocument().createElement(SDLIP.SearchResult);
      Element eDocProps = XMLObject.create(docProps).getElement();
      service.getDocs(eSearchResult, new RangeEnumerator(docSelection), eDocProps);

      return eSearchResult;
    }

    public Timer getTimer() {
      return timer;
    }

    public boolean isStateful() {
      return timer != null;
    }

    public int getSID() {
      return SID;
    }

    public int getNumDocs() {
      return service.getNumDocs();
    }

    public void cancel() {
      service.interrupt();
      timer.interrupt();
    }

    public String toString() {
      return "[SID:" + SID + "]";
    }
  }

  static final String SUBCOLNAME = "Britannica";
  static final int MAX_STATE_TIMEOUT = 10 * 60; // 10 min

  Hashtable sessions = new Hashtable(); // Integer -> Session
  int lastSessionID = 0;
  DBG dbg = new DBG();

  public BritannicaLSP() {
  }

  public void setDBG(DBG dbg) {
    this.dbg = dbg;
  }

  public void search(
		     int clientSID,
		     sdlip.XMLObject subcollections,
		     sdlip.XMLObject query,
		     int numDocs,
		     sdlip.XMLObject docProps,
		     int stateTimeoutReq,
		     sdlip.XMLObject queryOptions,
		     IntHolder expectedTotal,
		     IntHolder stateTimeout,
		     IntHolder serverSID,
		     sdlip.XMLObject serverDelegate,
		     sdlip.XMLObject result
		     )
    throws SDLIPException {
      
    /* makes sure that the XMLObject passed into query is
     * a DOM-enabled sdlip.xml.dom.XMLObject and convert it if not */
    Element eQuery = XMLObject.create(query).getElement();
      
    // only DAV:basicsearch is supported
    if(eQuery == null || !DAV.basicsearch.equals(eQuery.getTagName()))
      throw new SDLIPException(SDLIPException.BAD_QUERY_EXC, "Only DAV:basicsearch query type is supported");
    
    // only DAV:contains is supported
    String qstr = DOMUtil.getDescendantText(eQuery, DAV.contains);
    if(qstr == null) {

      eQuery = QueryUtil.canonicalDNF(eQuery);
      if(QueryUtil.isTrue(eQuery))
	throw new SDLIPException(SDLIPException.BAD_QUERY_EXC, "Query is always true!");
      if(QueryUtil.isFalse(eQuery)) {
	// empty set
	expectedTotal.value = 0;
	result.setString("<SearchResult xmlns='" + SDLIP.Namespace + "'/>");
	return;
      }
 
      NodeList eqList = eQuery.getElementsByTagName(DAV.eq);
      System.err.println("EQS:" + eqList.getLength());
      qstr = (eqList.getLength() == 1) ? DOMUtil.getChildText((Element)eqList.item(0), DAV.literal) : null;
      if(qstr == null)
	throw new SDLIPException(SDLIPException.BAD_QUERY_EXC, "Supported DAV:basicsearch queries are either DAV:contains or a single constraint on the name");
    }

    dbg.println(DBG.SHORT, "QUERY: " + qstr + ", numDocs: " + numDocs);
    
    // create a new session
    Session session = new Session(++lastSessionID, qstr, Math.min(stateTimeoutReq, MAX_STATE_TIMEOUT), this);

    if(session.isStateful()) {
      // stateful request, save the new session
      addSession(session);
    }
    String docSelection = (numDocs == sdlip.Search.RETURN_ALL_DOCS) ? "1-" : "1-" + numDocs;

    /* deliver the result */
    XMLObject.setElement(result, session.getDocs(docSelection, docProps));
    // set expected total number of results
    expectedTotal.value = session.getNumDocs();
    // set timeout after request processing for more accurate result
    if(session.isStateful()) {
      stateTimeout.value = session.getTimer().getTimeout();
      serverSID.value = session.getSID();
    }
  }

  /* Method of Timeoutable */

  public void timeout(Object handle) {
    
    Session session = (Session)handle;
    dbg.println(DBG.SHORT, "Session " + session + " timed out!");
    removeSession(session);
  }

  /* Methods of the source metadata interface */

  public void getInterface(sdlip.XMLObject version) throws SDLIPException {

    version.setString("<SDLIPInterface xmlns='" + SDLIP.Namespace + "'>" +
		      "  <SearchInterface/>" +
		      "  <MetadataInterface/>" +
		      "  <ResultAccessInterface/>" +
		      "</SDLIPInterface>");
  }

  public void getSubcollectionInfo(sdlip.XMLObject subcolInfo) throws SDLIPException {
    
    subcolInfo.setString("<subcolInfo xmlns='" + SDLIP.Namespace + "'" +
			 "            xmlns:d='DAV:'>" + 
			 "  <subcol>" +
			 "    <subcolName>" + SUBCOLNAME + "</subcolName>" +
			 "    <subcolDesc>" +
			 "       This collection delivers encyclopedia britannica articles." +
			 "       It supports only DAV:contains operator of DAV:basicsearch query language." +
			 "    </subcolDesc>" +
			 "    <defaultSubcol/>" +
			 "    <queryLangs>" +
			 "      <d:basicsearch/>" +
			 "    </queryLangs>" +
			 "  </subcol>" +
			 "</subcolInfo>");
  }

  public void getPropertyInfo(String subcolName, sdlip.XMLObject propInfo) throws SDLIPException {

    if(!SUBCOLNAME.equals(subcolName))
      throw new SDLIPException(SDLIPException.INVALID_SUBCOLLECTION_EXC, "The only subcollection available is '" + SUBCOLNAME + "'");

    propInfo.setString("<propList xmlns='" + SDLIP.Namespace + "'" + 
		       "          xmlns:s='" + Britannica.Namespace + "'>" +
		       "  <s:query><searchable/></s:query>" +
		       "  <s:title><retrievable/></s:title>" +
		       "  <s:article><retrievable/></s:article>" +
		       "  <s:abstract><retrievable/></s:abstract>" +
		       "</propList>");
  }

  // Methods of the ResultAccess interface

  // ResultAccess implementation

  public void cancelRequest(
		     int serverSID,
		     int reqID
		     )
    throws SDLIPException {

    // we can only cancel the whole session ;(
    Session s = findSession(serverSID);
    dbg.println(DBG.SHORT, "Cancelling session " + s);
    s.cancel();
    if(reqID == ResultAccess.CANCEL_ALL_REQUESTS) {
      removeSession(s);
    }
  }

  public void getSessionInfo(
			     int serverSID,
			     IntHolder expectedTotal,
			     IntHolder stateTimeout
			     )
    throws SDLIPException {
    

    Session session = findSession(serverSID);
    expectedTotal.value = session.getNumDocs();
    stateTimeout.value = session.getTimer().getTimeout();
    dbg.println(DBG.SHORT, "Info requested for " + session + ": total=" + expectedTotal.value + ", timeout=" + stateTimeout.value);
  }
  
  public void getDocs(
		      int serverSID,
		      int reqID,
		      sdlip.XMLObject docPropList,
		      String docsToGet,
		      sdlip.XMLObject res )
    throws SDLIPException {

    Session session = findSession(serverSID);
    dbg.println(DBG.SHORT, "Getting documents " + docsToGet + " for " + session);
    XMLObject.setElement(res, session.getDocs(docsToGet, docPropList));
  }
  
  public void extendStateTimeout(
				 int serverSID,
				 int additionalTime,
				 IntHolder timeAllotted )
    throws SDLIPException {

    Session session = findSession(serverSID);
    Timer timer = session.getTimer();
    timer.setTimeout( Math.min(timer.getTimeout() + additionalTime, MAX_STATE_TIMEOUT) );
    timeAllotted.value = timer.getTimeout();
    dbg.println(DBG.SHORT, "Extended state timeout of " + session + " for " + timeAllotted.value);
  }
  
  // Private methods

  void addSession(Session s) {
    sessions.put(new Integer(s.getSID()), s);
    dbg.println(DBG.SHORT, "New session " + s + " created");
  }
  
  void removeSession(Session s) {
    s.cancel();
    sessions.remove(new Integer(s.getSID()));
    dbg.println(DBG.SHORT, "Removing session " + s + ", " + sessions.size() + " left");
  }
  
  Session findSession(int SID) throws SDLIPException {
    Session s = (Session)sessions.get(new Integer(SID));
    if(s == null)
      throw new SDLIPException(SDLIPException.REQUEST_TIMEOUT_EXC, "Invalid or timed out session");
    return s;
  }

  public String toString() {
    return "LSP serving www.eb.com (Encyclopedia Britannica)";
  }


  public static void main(String args[]) throws Exception {
    
    /* Instantiate an SDLIP/DASL transport module
     * This also starts the server on the given port */

    if(args.length != 1) {
      System.err.println("Usage: java BritannicaLSP <port>");
      System.exit(1);
    }

    BritannicaLSP lsp = new BritannicaLSP();
    DBG dbg = new DBG(DBG.SHORT);
    lsp.setDBG(dbg);

    ServerDaslTransport t1 = new ServerDaslTransport( Integer.parseInt(args[0]) );
    t1.setDBG(dbg);
    t1.addLSP(lsp);
    
    /* Initialize CORBA transport only if runtime libraries are available */
    try {
      ServerCorbaTransport t2 = new ServerCorbaTransport();
      t2.setDBG(dbg);
      t2.addLSP(lsp);
    } catch (NoClassDefFoundError err) {}
  }

}

