package sdlip.examples;

import sdlip.SDLIPException;
/* holder for integer values */
import org.omg.CORBA.IntHolder;
/* transport module libraries */
import sdlip.helpers.*;
/* following packages are imported only if DOM-aware XMLObjects are used */
import sdlip.xml.dom.*;
import org.w3c.dom.*;
/* SDLIP XML element names */
import sdlip.SDLIP;

import java.applet.*;
import netscape.javascript.*;

/**
 * This is an example application included in the SDLIP distribution.
 * <p>Usage: <tt>java sdlip.examples.Client &lt;URL_or_IOR> &lt;method> ...method parameters...</tt>
 * <p>Allows to invoke all methods defined in the Search and ResultAccess interfaces on an LSP using a command line 
 * tool. The <tt>&lt;method></tt> parameter corresponds to the name of the method to be invoked. Following methods are 
 * supported:
 * <p><pre>
 *   search              &lt;clientSID> &lt;queryStr> &lt;numDocs> &lt;stateTimeoutReq>
 *   getSessionInfo      &lt;serverSID>
 *   getDocs             &lt;serverSID> &lt;reqID> &lt;docsToGet>
 *   extendStateTimeout  &lt;serverSID> &lt;additionalTime>
 *   cancelRequest       &lt;serverSID> &lt;reqID>
 * </pre>
 */

public class ClientApp extends Applet {

  JSObject win;
  final String PRIVILEGE = "UniversalConnect"; // "Netcaster";

  public void init() {
    super.init();

    try {
      netscape.security.PrivilegeManager.enablePrivilege(PRIVILEGE);
      //    netscape.security.PrivilegeManager.enablePrivilege("UniversalConnect");
      
      System.out.println("[ClientApp] started");
      showStatus("GUI Library Client ready!");

    } catch (Exception any) {
      System.err.println("[ClientApp] could not start: " + any);
    }
  }

  static String expectedTotalString(int total) {
    
    if(total == sdlip.Search.UNKNOWABLE)
      return "unknowable";
    else if(total == sdlip.Search.NOT_YET_KNOWN)
      return "not yet known";
    else
      return String.valueOf(total);
  }

  void display(String mode, String text) {
    Object[] buffer = new Object[2];
    buffer[0] = mode;
    buffer[1] = text;
    //    System.err.println("Writing to " + win);
    win.call("display", buffer);
  }

  int getNSEnd(String name) {
    // first try to detemine namespace directly
    int l = name.length()-1;
    do {
      char c = name.charAt(l);
      if(c == '#' || c == ':' || c == '/')
	break;
      l--;
    } while (l >= 0);
    l++;
    return l;
  }

  String getNS(String name) {
    int l = getNSEnd(name);
    if(l > 0) {
      return name.substring(0, l);
    } else
      return "";
  }

  String getName(String name) {
    int l = getNSEnd(name);
    if(l > 0) {
      return name.substring(l);
    } else
      return name;
  }

  String SID;

  public String getSID() {
    return SID;
  }


  public String search(JSObject win, String url, String query, String limit, String timeoutReq) {

    this.win = win;

    try {
      netscape.security.PrivilegeManager.enablePrivilege(PRIVILEGE);

      //      System.err.println("SEARCH: REQ " + System.currentTimeMillis());

      doSearch(url, query, limit, timeoutReq);
    } catch (Exception any) {
      procExc("Search", any);
    }
    return null;
  }

  public String getDocs(JSObject win, String url, String sid, String reqID, String docsToGet) {

    this.win = win;

    try {
      netscape.security.PrivilegeManager.enablePrivilege(PRIVILEGE);

      doGetDocs(url, sid, reqID, docsToGet);
    } catch (Exception any) {
      procExc("getDocs", any);
    }
    return null;
  }

  public String discoverMetadata(JSObject win, String url) {
    
    this.win = win;
    try {
      netscape.security.PrivilegeManager.enablePrivilege(PRIVILEGE);
      
      doDiscoverMetadata(url);
    } catch (Exception any) {
      procExc("Metadata", any);
    }
    return null;
  }

  public String getSessionInfo(JSObject win, String url, String SID) {
    
    this.win = win;
    try {
      netscape.security.PrivilegeManager.enablePrivilege(PRIVILEGE);
      
      doGetSessionInfo(url, SID);
    } catch (Exception any) {
      procExc("getSessionInfo", any);
    }
    return null;
  }

  public String extendStateTimeout(JSObject win, String url, String SID, String addTime) {
    
    this.win = win;
    try {
      netscape.security.PrivilegeManager.enablePrivilege(PRIVILEGE);
      
      doExtendStateTimeout(url, SID, addTime);
    } catch (Exception any) {
      procExc("extendStateTimeout", any);
    }
    return null;
  }

  public String cancelRequest(JSObject win, String url, String SID, String reqID) {
    
    this.win = win;
    try {
      netscape.security.PrivilegeManager.enablePrivilege(PRIVILEGE);
      
      doCancelRequest(url, SID, reqID);
    } catch (Exception any) {
      procExc("cancelRequest", any);
    }
    return null;
  }

  void procExc(String method, Exception exc) {
    try {
      String msg = null;
      if(exc instanceof sdlip.SDLIPException) {
	sdlip.SDLIPException sdlip = (sdlip.SDLIPException)exc;
	msg = "SDLIP exception (" + sdlip.getCode() + "): " + sdlip.getReason();
      } else
	msg = "Could not perform method: " + method + ", reason: " + exc.getMessage();
      System.err.println(msg);
      display("err", msg);
    } catch (Exception any) {}
  }
  
  DBG dbg = new DBG(DBG.SHORT);

  private String replace( String src, String in, String out ) {
    int pos = src.indexOf(in);
    
    while (pos != -1) {
      src = src.substring(0,pos) + out + src.substring(pos+in.length());
      pos = src.indexOf(in);
    }   
    return src;
  }

  void doGetSessionInfo(String url, String sid) throws Exception {

    StringBuffer b = new StringBuffer("<h2>Session info for " + url + ", SID:" + sid + "</h2>");

    ClientTransportModule tm = ClientTransportModule.create(url);
    tm.setDBG(dbg);

    IntHolder hExpectedTotal = new IntHolder();
    IntHolder hStateTimeout = new IntHolder();

    tm.getSessionInfo(Integer.parseInt(sid), hExpectedTotal, hStateTimeout);

    b.append("<p>EXPECTED TOTAL: " + expectedTotalString(hExpectedTotal.value));
    b.append("<p>TIMEOUT: " + hStateTimeout.value);
    
    display("results", b.toString());
  }

  void doCancelRequest(String url, String sid, String reqID) throws Exception {

    StringBuffer b = new StringBuffer("<h2>Result from " + url + ", SID:" + sid + "</h2>");

    ClientTransportModule tm = ClientTransportModule.create(url);
    tm.setDBG(dbg);
    
    tm.cancelRequest(Integer.parseInt(sid), Integer.parseInt(reqID));

    if(Integer.parseInt(reqID) == 0)
      b.append("<p>Session canceled");
    else
      b.append("<p>Request canceled");
    
    display("results", b.toString());
  }

  void doExtendStateTimeout(String url, String sid, String addTime) throws Exception {

    StringBuffer b = new StringBuffer("<h2>" + url + ", SID:" + sid + "</h2>");
    
    ClientTransportModule tm = ClientTransportModule.create(url);
    tm.setDBG(dbg);
    
    IntHolder hTimeAllotted = new IntHolder();
    
    tm.extendStateTimeout(Integer.parseInt(sid),
			  Integer.parseInt(addTime),
			  hTimeAllotted);
    
    b.append("<p>TIME ALLOTTED: " + hTimeAllotted.value);
    
    display("results", b.toString());
  }

  void doDiscoverMetadata(String url) throws Exception {

    StringBuffer b = new StringBuffer("<h2>Metadata for " + url + "</h2>");

    ClientTransportModule tm = ClientTransportModule.create(url);
    tm.setDBG(dbg);

    /* the result will be held in a DOM-enabled SDLIP.xml.dom.XMLObject */
    XMLObject result = new XMLObject();
    
    /* Metadata */
    
    tm.getInterface(result);
    b.append("<h3>Supported Interfaces</h3><pre>" + replace(result.getPrettyString(), "<", "&lt;") + "</pre>");
    
    tm.getSubcollectionInfo(result);
    b.append("<h3>Subcollections</h3><pre>" + replace(result.getPrettyString(), "<", "&lt;") + "</pre>");
    
    /* get a DOM element from an XMLObject */
    Element subcols = result.getElement();
    
    /* iterate over all found subcollections */
    NodeList colList = DOMUtil.getChildElements(subcols, SDLIP.subcol);
    for(int i=0; i < colList.getLength(); i++) {
      
      Element eCol = (Element)colList.item(i);
      String colName = DOMUtil.getChildText(eCol, SDLIP.subcolName);
      
      /* retrieve properties of the subcollection */
      tm.getPropertyInfo(colName, result);
      b.append("<h3>Properties of " + colName + "</h3><pre>" + replace(result.getPrettyString(), "<", "&lt;") + "</pre>");
    }
    display("results", b.toString());
  }

  void printResults(StringBuffer b, XMLObject result) {
    /* get the top XML element from an XMLObject */
    Element searchResult = result.getElement();
    
    b.append("<ul>");
    /* iterate over all found documents */
    NodeList docList = DOMUtil.getChildElements(searchResult, SDLIP.doc);
    for(int i=0; i < docList.getLength(); i++) {
      
      /* print document ID */
      Element eDoc = (Element)docList.item(i);
      b.append("<li><b>[" + DOMUtil.getChildText(eDoc, SDLIP.DID) + "]</b>");
      b.append("<br><table>");
      
      /* print all document properties in format NAME = "VALUE" */
      Element ePropList = DOMUtil.getChild(eDoc, SDLIP.propList);
      NodeList propList = DOMUtil.getChildElements(ePropList, "*");
      for(int j=0; j < propList.getLength(); j++) {
	Element eProp = (Element)propList.item(j);
	String tagName = eProp.getTagName();
	String propValue = DOMUtil.getText(eProp);
	if(propValue != null && propValue.startsWith("http:"))
	  propValue = "<a href='" + propValue + "'>" + propValue + "</a>";
	b.append("<tr><td><a href='" + tagName + "'><b>" +
		 "<font color='darkblue'>" + getName(tagName) + "</font></a></td><td>" + propValue + "</td></tr>");
      }
      b.append("</table></li>");
    }
    b.append("</ul>");
    
    //      b.append("<p>RESULT:\n" + result.getPrettyString());
  }

  void doSearch(String url, String queryStr, String limit, String timeoutReq) throws Exception {

    StringBuffer b = new StringBuffer("<h2>Search Results</h2>");

    ClientTransportModule tm = ClientTransportModule.create(url);
    tm.setDBG(dbg);
    
    String clientSID = "12345";
    String numDocs = !"".equals(limit) ? limit : "10";
    
    /* the result will be held in a DOM-enabled sdlip.xml.dom.XMLObject */
    XMLObject result = new XMLObject();
    IntHolder hTotal = new IntHolder();
    IntHolder hTimeout = new IntHolder();
    IntHolder hServerSID = new IntHolder();
    XMLObject delegate = new XMLObject();
    
    // if a string is supplied, wrap it into DAV:contains
    queryStr = queryStr.trim();
    if(!queryStr.startsWith("<"))
      queryStr = "<basicsearch xmlns='DAV:'>" +
	"  <where><contains>" + queryStr + "</contains></where>" +
	"</basicsearch>";

    tm.search(Integer.parseInt(clientSID),  /* some client SID */
	      null,            /* search the default subcollection */
	      new XMLObject(queryStr),
	      Integer.parseInt(numDocs),
	      null,            /* retrieve all properties */
	      Integer.parseInt(timeoutReq), /* maintain state */
	      null,            /* no query options */
	      hTotal,          /* expected total number of results */
	      hTimeout,        /* place to return stateTimeout */
	      hServerSID,      /* place to return serverSID - don't care */
	      delegate,        /* place to return serverDelegate - don't care */
	      result           /* place to return search result */
	      );
    
    printResults(b, result);
    if(delegate.getElement() != null)
      b.append("<p>DELEGATE:\n" + delegate.getPrettyString());
    b.append("<p>EXPECTED TOTAL: " + expectedTotalString(hTotal.value));
    b.append("<p>TIMEOUT: " + hTimeout.value);
    b.append("<p>SERVER SID: " + hServerSID.value);
    SID = String.valueOf(hServerSID.value);
    
    display("results", b.toString());
    
  }

  void doGetDocs(String url, String serverSID, String reqID, String docsToGet) throws Exception {

    StringBuffer b = new StringBuffer("<h2>Results " + docsToGet + " from " + url + ", SID: " + serverSID + "</h2>");

    ClientTransportModule tm = ClientTransportModule.create(url);
    tm.setDBG(dbg);
    
    XMLObject result = new XMLObject();

    tm.getDocs(Integer.parseInt(serverSID),
	       Integer.parseInt(reqID),
	       null,
	       docsToGet,
	       result);

    printResults(b, result);
    
    display("results", b.toString());
    
  }

}
