package sdlip.examples;

import sdlip.helpers.*;

/**
 * This application starts both the SimpleLSP application described in the SDLIP Java tutorial
 * and the SwitchBoardLSP.<p>
 * The SimpleLSP is exported under the name "/simplelsp".
 * The SwitchBoardLLSP is available under "/peoplelookup".
 */

public class ComplexLSP {

  public static void main(String[] args) throws Exception {
    
    if(args.length != 1) {
      System.err.println("Usage: java sdlip.examples.ComplexLSP <port>");
      System.exit(1);
    }

    SimpleLSP lsp1 = new SimpleLSP();
    SwitchBoardLSP lsp2 = new SwitchBoardLSP();

    //    Z39LSP lsp3 = new Z39LSP("ibm2.loc.gov:2210");
    Z39LSP lsp3 = new Z39LSP("z3950.loc.gov:7090");
    lsp3.setDefaultSubcol("VOYAGER");

    Z39LSP lsp4 = new Z39LSP("tikal.dev.oclc.org");

    Z39LSP lsp5 = new Z39LSP("melvyl.ucop.edu");
    lsp5.setDefaultSubcol("CATALOG");

    BritannicaLSP lsp6 = new BritannicaLSP();

    DBG dbg = new DBG(DBG.SHORT);
    lsp2.setDBG(dbg);
    lsp3.setDBG(dbg);
    lsp4.setDBG(dbg);
    lsp5.setDBG(dbg);

    ServerDaslTransport t1 = new ServerDaslTransport( Integer.parseInt(args[0]) );
    t1.setDBG(dbg);
    t1.addLSP("/simplelsp", lsp1);
    t1.addLSP("/peoplelookup", lsp2);
    t1.addLSP("/lc", lsp3);
    t1.addLSP("/oclc", lsp4);
    t1.addLSP("/cdl", lsp5);
    t1.addLSP("/britannica", lsp6);
    try {
      ServerCorbaTransport t2 = new ServerCorbaTransport();
      t2.setDBG(dbg);
      t2.addLSP("/simplelsp", lsp1);
      t2.addLSP("/peoplelookup", lsp2);
      t2.addLSP("/lc", lsp3);
      t2.addLSP("/oclc", lsp4);
      t2.addLSP("/cdl", lsp5);
      t2.addLSP("/britannica", lsp6);
    } catch (NoClassDefFoundError err) {}
  }
}
