package sdlip.examples;

import sdlip.SDLIPException;
import org.omg.CORBA.IntHolder;
/* transport module libraries */
import sdlip.helpers.*;
/* following packages are imported only if DOM-aware XMLObjects are used */
import sdlip.xml.dom.*;
import org.w3c.dom.*;

/* SDLIP XML element names */
import sdlip.SDLIP;

/**
 * This is an example application included in the SDLIP distribution.
 * <p>Usage: <tt>java sdlip.examples.DiscoverMetadata &lt;URL_or_IOR></tt>
 * <p>Fetches and prints out the complete metadata exposed by the LSP.
 */

public class DiscoverMetadata {

  public static void main(String args[]) {

    if(args.length == 0) {
      System.err.println("Usage: java sdlip.examples.DiscoverMetadata <URL_or_IOR> [<subcol>]");
      System.exit(1);
    }

    try {
      String searchServerURI = args[0];

      String subcolName = null;
      if(args.length > 1)
	subcolName = args[1];

      /* an SDLIP/CORBA or SDLIP/DASL transport module is created
       * depending on whether a URL or an IOR is supplied */
      ClientTransportModule tm = ClientTransportModule.create(searchServerURI);
      tm.setDBG(new DBG(DBG.VERBOSE));

      /* the result will be held in a DOM-enabled SDLIP.xml.dom.XMLObject */
      XMLObject result = new XMLObject();


      /* Metadata */

      if(subcolName != null) {

	/* just retrieve properties of the subcollection */
	tm.getPropertyInfo(subcolName, result);
	System.out.println("\nPROPERTIES of " + subcolName + ":\n\n" + result.getPrettyString());

      } else {
	
	//	tm.getInterface(result);
	//	System.out.println("\nSUPPORTED INTERFACES:\n\n" + result.getPrettyString());
	
	tm.getSubcollectionInfo(result);
	System.out.println("\nSUBCOLLECTIONS:\n\n" + result.getPrettyString());
	
	/* get a DOM element from an XMLObject */
	Element subcols = result.getElement();

	/* iterate over all found subcollections */
	NodeList colList = DOMUtil.getChildElements(subcols, SDLIP.subcol);
	for(int i=0; i < colList.getLength(); i++) {
	  
	  Element eCol = (Element)colList.item(i);
	  String colName = DOMUtil.getChildText(eCol, SDLIP.subcolName);
	  
	  /* retrieve properties of the subcollection */
	  //	  System.getProperties().put("sdlip.debug", "on");
	  tm.getPropertyInfo(colName.substring(0, colName.length()-1), result);
	  System.out.println("\nPROPERTIES of " + colName + ":\n\n" + result.getPrettyString());
	}
      }

    } catch (SDLIPException sdlip) {
      System.err.println("SDLIP exception (" + sdlip.getCode() + "): " + sdlip.getReason());
    }
  }
}
