package sdlip.examples;

import sdlip.SDLIPException;
/* transport module libraries */
import sdlip.helpers.*;
/* DOM-enabled XMLObject */
import sdlip.xml.dom.XMLObject;
/* holder for integer values */
import org.omg.CORBA.IntHolder;

import sdlip.SDLIP;
import sdlip.DublinCore;

/**
 * This is the SimpleLSP application described in the SDLIP Java tutorial.
 */

public class SimpleLSP implements sdlip.Search, sdlip.Metadata {

  public void search(
		     int clientSID,
		     sdlip.XMLObject subcollections,
		     sdlip.XMLObject query,
		     int numDocs,
		     sdlip.XMLObject docProps,
		     int stateTimeoutReq,
		     sdlip.XMLObject queryOptions,
		     IntHolder expectedTotal,
		     IntHolder stateTimeout,
		     IntHolder serverSID,
		     sdlip.XMLObject serverDelegate,
		     sdlip.XMLObject result
		     )
    throws SDLIPException {
      
    /* makes sure that the XMLObject passed into query is
     * a DOM-enabled sdlip.xml.dom.XMLObject and convert it if not */
    System.out.println("Query is:\n" + XMLObject.create(query).getPrettyString());

    /* deliver the result; in a real LSP this would access the underlying collection */
    result.setString("<SearchResult xmlns='" + SDLIP.Namespace + "' " +
		     "              xmlns:dc='" + DublinCore.Namespace + "'>" +
		     "  <doc>" +
		     "    <DID>1</DID>" +
		     "    <propList>" +
		     "      <dc:Title>Fighting a Wind Mill</dc:Title>" +
		     "      <dc:Creator>Don Quichote</dc:Creator>" +
		     "    </propList>" +
		     "  </doc>" +
		     "  <doc>" +
		     "    <DID>2</DID>" +
		     "    <propList>" +
		     "      <dc:Title>Document Without Creator</dc:Title>" +
		     "    </propList>" +
		     "  </doc>" +
		     "</SearchResult>");

    /* set the total expected number of results */
    expectedTotal.value = 2;
  }

  /* Methods of the source metadata interface */

  public void getInterface(sdlip.XMLObject version) throws SDLIPException {

    version.setString("<SDLIPInterface xmlns='" + SDLIP.Namespace + "'>" +
		      "  <SearchInterface/>" +
		      "  <MetadataInterface/>" +
		      "</SDLIPInterface>");
  }

  public void getSubcollectionInfo(sdlip.XMLObject subcolInfo) throws SDLIPException {
    
    subcolInfo.setString("<subcolInfo xmlns='" + SDLIP.Namespace + "'>" +
			 "  <subcol>" +
			 "    <subcolName>Dummy-Collection</subcolName>" +
			 "    <subcolDesc>This collection always returns two results independent from the stated query</subcolDesc>" +
			 "    <defaultSubcol/>" +
			 "    <queryLangs/>" +
			 "  </subcol>" +
			 "</subcolInfo>");
  }

  public void getPropertyInfo(String subcolName, sdlip.XMLObject propInfo) throws SDLIPException {

    if(!"Dummy-Collection".equals(subcolName))
      throw new SDLIPException(SDLIPException.INVALID_SUBCOLLECTION_EXC, "The only subcollection available is 'Dummy-Collection'");

    propInfo.setString("<propList xmlns='" + SDLIP.Namespace + "'" + 
		       "          xmlns:dc='" + DublinCore.Namespace + "'>" +
		       "  <dc:Title><retrievable/></dc:Title>" +
		       "  <dc:Creator><retrievable/></dc:Creator>" +
		       "</propList>");
  }

  public static void main(String args[]) throws Exception {
    
    /* Instantiate an SDLIP/DASL transport module
     * This also starts the server on the given port */

    if(args.length != 1) {
      System.err.println("Usage: java sdlip.examples.SimpleLSP <port>");
      System.exit(1);
    }
    
    SimpleLSP lsp = new SimpleLSP();
    DBG dbg = new DBG(DBG.SHORT);
    
    ServerDaslTransport t1 = new ServerDaslTransport( Integer.parseInt(args[0]) );
    t1.setDBG(dbg);
    t1.addLSP(lsp);
    
    /* Initialize CORBA transport only if runtime libraries are available */
    try {
      ServerCorbaTransport t2 = new ServerCorbaTransport();
      t2.setDBG(dbg);
      t2.addLSP(lsp);
    } catch (NoClassDefFoundError err) {}
  }
}

