package sdlip.examples.demo;

import sdlip.SDLIPException;
/* holder for integer values */
import org.omg.CORBA.IntHolder;
/* transport module libraries */
import sdlip.helpers.*;
/* following packages are imported only if DOM-aware XMLObjects are used */
import sdlip.xml.dom.*;
import org.w3c.dom.*;
/* SDLIP XML element names */
import sdlip.SDLIP;
import sdlip.DAV;

import netscape.javascript.*;

import java.util.*;

/**
 * A TreeNode representing an SDLIP collection.
 */

public class CollectionNode extends BasicNode {

  Element metadata;
  Element properties;
  Vector attributes;
  ServerNode server;
  String colName, colDesc;
  Hashtable attributesToRetrieve = new Hashtable(); // of NestedAttributes
  boolean isDefault;

  public CollectionNode(Element c, ServerNode server, boolean isDefault) throws Exception {

    super();
    this.server = server;
    this.metadata = c;
    this.isDefault = isDefault;
    // trimming should not be necessary, just to be more robust
    this.colName = trim(DOMUtil.getChildText(c, SDLIP.subcolName));
    this.colDesc = DOMUtil.getChildText(c, SDLIP.subcolDesc);
    this.label = "<b><a href='javascript:parent.frmTop.document.clientApp.displaySearchForm(" + this.hashCode() + ")'><font color=darkred>" + colName + "</font></a></b>";
    if(isDefault)
      label += " <small>(default)</small>";
    //    System.out.println("ADDING collection " + label);
  }

  public ServerNode getServer() {
    return server;
  }

  public Enumeration getAttributes() {

    return attributes.elements();
  }

  public boolean isDefault() {

    return isDefault;
  }

  public String getName() {
    return colName;
  }

  public String getDesc() {
    return colDesc;
  }

  public void getMetadata() throws Exception {

    ClientTransportModule tm = ClientTransportModule.create(server.getHost());
    tm.setDBG(new DBG(DBG.VERBOSE));

    /* the result will be held in a DOM-enabled SDLIP.xml.dom.XMLObject */
    XMLObject result = new XMLObject();

    /* retrieve properties of the subcollection */
    tm.getPropertyInfo(colName, result);
    //    b.append("<h3>Properties of " + colName + "</h3><pre>" + replace(result.getPrettyString(), "<", "&lt;") + "</pre>");
    properties = result.getElement();
    attributes = NestedAttribute.collectAttributes(properties);
  }

  public void displaySearchForm(StringBuffer b, String title, String buttonText, boolean search, String callbackMethod, int hashCode) throws Exception {

    if(properties == null) {
      server.getClient().setStatus("Retrieving collection metadata...");
      getMetadata();
    }
    b.append("<h2>" + title + " for collection <font color=darkblue>" + getName() + "</font></h2>");

    if(colDesc != null) {
      b.append(colDesc).append("<p>");
    }

    b.append("<form name='formSearch' onSubmit='javascript:parent.frmTop.document.clientApp." + callbackMethod + "(" + hashCode + ",document.formSearch); return false'><table border=1 cellspacing=0 cellpadding=3>");
    b.append("<tr bgcolor='eeeeff'><td align=center><b><small>Attribute Name</td><td align=center><b><small>Retrieve?</td>");
    if(search) {
      b.append("<td align=center><b><small>Search Value</td>");
    }
    b.append("<td align=center><b><small>Description</td></tr>");
    //    b.append(DOMWriter.prettyPrint(properties));


    for(Enumeration en = attributes.elements(); en.hasMoreElements(); ) {

      NestedAttribute a = (NestedAttribute)en.nextElement();
      if(!search && !a.isRetrievable())
	continue;

      b.append("<tr><td bgcolor='eeffee'><b>" + a.getPropName() + "</td><td align=center>");

      // retrievable?
      if(a.isRetrievable()) {
	b.append("<input type='checkbox' name='retrieve_" + a.getPropName() + "'");
	if(attributesToRetrieve.contains(a))
	  b.append(" CHECKED");
	b.append(">");
      }

      // searchable?
      if(search) {
	b.append("&nbsp;</td><td>");
	if(a.isSearchable())
	  b.append("<input type='text' name='search_" + a.getPropName() + "' value='' size=30>");
      }

      b.append("&nbsp;</td><td><small>");
      b.append(a.getPropDesc());
      b.append("&nbsp;</td></tr>");
    }
    //    b.append("<tr><td>&nbsp;<td><input type='submit' value='Submit Query'></tr>");
    b.append("</table>");
    b.append("<br><input type='submit' value='" + buttonText + "'>");
    b.append("</form>");
  }


  public void makeAttributeChoice(JSObject form) throws Exception {

    attributesToRetrieve.clear();
    for(Enumeration en = attributes.elements(); en.hasMoreElements(); ) {
      
      NestedAttribute a = (NestedAttribute)en.nextElement();
      String name = a.getPropName();
      boolean retrieve = false;
      try {
	JSObject check = (JSObject) form.getMember("retrieve_" + name);
	retrieve = ((Boolean) check.getMember("checked")).booleanValue();
	if(retrieve)
	  attributesToRetrieve.put(a, a);
      } catch (Exception any) {}
    }
  }

  public Element getPropListToRetrieve() {

    // create propList to retrieve
    Element ePropList = XMLObject.getDocument().createElement(SDLIP.propList);
    int numPropsRetrieve = 0;

    for(Enumeration en = attributes.elements(); en.hasMoreElements(); ) {
      
      NestedAttribute a = (NestedAttribute)en.nextElement();
      if(attributesToRetrieve.contains(a)) {
	// add to propList
	ePropList.appendChild(a.getNestedElements());
	numPropsRetrieve++;
      }
    }

    return (numPropsRetrieve == 0) ? null : ePropList;
  }

    public static String trim(String str) {
		return trimLeft(trimRight(str));
    }
    
    public static String trimLeft(String str) {
    	if(str == null)
    		return null;
    	int i=0;
    	while ( i<str.length() && Character.isSpace(str.charAt(i)) )
    		i++;
    	return i < str.length() ? str.substring(i) : "";
    }
    
    public static String trimRight(String str) {
    	if(str == null)
    		return null;
    	int i = str.length()-1;
    	while ( i >= 0 && Character.isSpace(str.charAt(i)) )
    		i--;
    	return i >= 0 ? str.substring(0, i+1) : "";
    }




  public ResultNode search(CollectionNode n, JSObject form) throws Exception {

    //    StringBuffer b = new StringBuffer("<h2>Search Results</h2>");

    // create DAV basicsearch query
    Document doc = XMLObject.getDocument();
    Element eBasicsearch = doc.createElement(DAV.basicsearch);
    Element eWhere = doc.createElement(DAV.where);
    Element eAnd = doc.createElement(DAV.and);


    eBasicsearch.appendChild(eWhere);

    int numQueryConstraints = 0;
    Element eLastConstraint = null;

    // construct result name
    int MAX_NAME_LEN = 20;
    String resultName = "";

    attributesToRetrieve.clear();
    for(Enumeration en = attributes.elements(); en.hasMoreElements(); ) {
      
      NestedAttribute a = (NestedAttribute)en.nextElement();
      String name = a.getPropName();
      String value = null;
      boolean retrieve = false;
      try {
	JSObject input = (JSObject) form.getMember("search_" + name);
	value = (String)input.getMember("value");
      } catch (Exception any) {}
      try {
	JSObject check = (JSObject) form.getMember("retrieve_" + name);
	retrieve = ((Boolean) check.getMember("checked")).booleanValue();
	if(retrieve)
	  attributesToRetrieve.put(a, a);
      } catch (Exception any) {}

      // add to basicsearch query
      if(value != null && !"".equals(value)) {
	Element eEq = doc.createElement(DAV.eq);
	Element eProp = doc.createElement(DAV.prop);
	eEq.appendChild(eProp);
	eProp.appendChild(a.getNestedElements());
	eEq.appendChild(DOMUtil.createTextNode(doc, DAV.literal, value));
	eLastConstraint = eEq;
	eAnd.appendChild(eEq);
	numQueryConstraints++;

	if(!"".equals(resultName))
	  resultName += " | ";
	resultName += value;
      }

      //      System.err.println("Attribute: " + name + ", value: " + value + ", retrieve: " + retrieve);
    }

    if(numQueryConstraints == 0) {
      throw new SDLIPException(SDLIPException.BAD_QUERY_EXC, "Empty query");
      
    } else if(numQueryConstraints == 1) {
      eWhere.appendChild(eLastConstraint);

    } else
      eWhere.appendChild(eAnd);

    //    System.err.println("QUERY: \n" + DOMWriter.prettyPrint(eBasicsearch));
    Element ePropList = getPropListToRetrieve();
    //    System.err.println("docProps: \n" + DOMWriter.prettyPrint(ePropList));

    if(resultName.length() > MAX_NAME_LEN)
      resultName = resultName.substring(0, MAX_NAME_LEN) + "...";

    ResultNode r = new ResultNode(this, resultName, eBasicsearch, ePropList);
    if(!r.isEmpty())
      addChild(r);
    return r;
  }


}
