package sdlip.helpers;

import java.io.*;
import java.net.*;

import sdlip.CORBA.*;
//import sdlip.xml.dom.*;
// import sdlip.http.*;
//import org.omg.CosNaming.*;

public class ClientCorbaTransport extends ClientTransportModule {

  static org.omg.CORBA.ORB orb;

  org.omg.CORBA.Object lsp;

  public ClientCorbaTransport() {
  }

  public ClientCorbaTransport(String uri) {
    setLspUri(uri);
  }

  //  static final String ORB_CLASS_PROP = "org.omg.CORBA.ORBSingletonClass";

  public void setLspUri(String uri) {

    if(orb == null) {
      if(!((new org.omg.CORBA.IntHolder()) instanceof org.omg.CORBA.portable.Streamable))
	throw new RuntimeException("The SDLIP/CORBA transport module requires that the org.omg.CORBA package appears in your CLASSPATH before the SDLIP package. Make sure corba-sdlip.jar goes in front of sdlip.jar.");

      orb = org.omg.CORBA.ORB.init(); // args,null);
    }

    if(uri.startsWith(sdlip.SDLIP.NameServerURISchema + ":")) {
      try {
	// resolve IOR
	String url = uri.substring(sdlip.SDLIP.NameServerURISchema.length() + 1);
	BufferedReader din = new BufferedReader(new InputStreamReader( new URL(url).openStream() ) );
	uri = din.readLine();
	//	System.err.println("URI is: " + uri);
	din.close();
      } catch (IOException ioex) {
	throw new RuntimeException(toString() + " could not resolve IOR: " + ioex.getMessage());
      }
    }

    lsp = orb.string_to_object(uri);
  }

  /**
   * Implementation of the search method of the Abstract Interface
   */
  public void search(
		     int clientSID,
		     sdlip.XMLObject subcols,
		     sdlip.XMLObject query,
		     int numDocs,
		     sdlip.XMLObject docProps,
		     int stateTimeoutReq,
		     sdlip.XMLObject queryOptions,
		     org.omg.CORBA.IntHolder expectedTotal,
		     org.omg.CORBA.IntHolder stateTimeout,
		     org.omg.CORBA.IntHolder serverSID,
		     sdlip.XMLObject serverDelegate,
		     sdlip.XMLObject result
		     )
    throws sdlip.SDLIPException {

    org.omg.CORBA.StringHolder hDelegate = new org.omg.CORBA.StringHolder();
    org.omg.CORBA.StringHolder hResult = new org.omg.CORBA.StringHolder();
    // default value for expectedTotal in case we have a stateless server
    expectedTotal.value = sdlip.Search.UNKNOWABLE;

    try {
      SearchHelper.narrow(lsp).search(clientSID,
				      toString(subcols),
				      toString(query),
				      numDocs,
				      toString(docProps),
				      stateTimeoutReq,
				      toString(queryOptions),
				      expectedTotal,
				      stateTimeout,
				      serverSID,
				      hDelegate,
				      hResult);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
    serverDelegate.setString(hDelegate.value);
    result.setString(hResult.value);
  }

  void raiseSDLIPExc(Exception ex) throws sdlip.SDLIPException {
    if(ex instanceof sdlip.CORBA.SDLIPException) {
      sdlip.CORBA.SDLIPException se = (sdlip.CORBA.SDLIPException)ex;
      sdlip.XMLObject xDetails = new SimpleXMLObject(se.details);
      throw new sdlip.SDLIPException(se.code, se.reason, xDetails);
    } else {
      dbg.println(DBG.SHORT, "Internal exception: " + ex);
      throw new sdlip.SDLIPException(sdlip.SDLIPException.SERVICE_UNAVAILABLE_EXC, String.valueOf(ex));
    }
  }

  String toString(sdlip.XMLObject obj) throws sdlip.SDLIPException {
    String res = obj == null ? "" : obj.getString();
    return res == null ? "" : res;
  }

  public void cancelRequest(
			    int serverSID,
			    int reqID
			    )
    throws sdlip.SDLIPException {

    try {
      ResultAccessHelper.narrow(lsp).cancelRequest(serverSID, reqID);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
  }

  public void getSessionInfo(
			     int serverSID,
			     org.omg.CORBA.IntHolder expectedTotal,
			     org.omg.CORBA.IntHolder stateTimeout
			     )
    throws sdlip.SDLIPException {

    try {
      ResultAccessHelper.narrow(lsp).getSessionInfo(serverSID, expectedTotal, stateTimeout);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
  }

  public void getDocs(
		      int serverSID,
		      int reqID,
		      sdlip.XMLObject docPropList,
		      String docsToGet,
		      sdlip.XMLObject res )
    throws sdlip.SDLIPException {

    org.omg.CORBA.StringHolder hResult = new org.omg.CORBA.StringHolder();
    try {
      ResultAccessHelper.narrow(lsp).getDocs(serverSID,
					     reqID,
					     toString(docPropList),
					     docsToGet,
					     hResult);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
    res.setString(hResult.value);
  }

  public void extendStateTimeout(
				 int serverSID,
				 int additionalTime,
				 org.omg.CORBA.IntHolder timeAllotted )
    throws sdlip.SDLIPException {

    try {
      ResultAccessHelper.narrow(lsp).extendStateTimeout(serverSID, additionalTime, timeAllotted);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
  }

  public void getInterface(sdlip.XMLObject intf) throws sdlip.SDLIPException {

    org.omg.CORBA.StringHolder hResult = new org.omg.CORBA.StringHolder();
    try {
      MetadataHelper.narrow(lsp).getInterface(hResult);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
    intf.setString(hResult.value);
  }

  public void getSubcollectionInfo(sdlip.XMLObject subcolInfo) throws sdlip.SDLIPException {

    org.omg.CORBA.StringHolder hResult = new org.omg.CORBA.StringHolder();
    try {
      MetadataHelper.narrow(lsp).getSubcollectionInfo(hResult);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
    subcolInfo.setString(hResult.value);
  }

  public void getPropertyInfo(String subcolName, 
			      sdlip.XMLObject propInfo) throws sdlip.SDLIPException {

    org.omg.CORBA.StringHolder hResult = new org.omg.CORBA.StringHolder();
    try {
      MetadataHelper.narrow(lsp).getPropertyInfo(subcolName == null ? "" : subcolName, hResult);
    } catch (Exception se) {
      raiseSDLIPExc(se);
    }
    propInfo.setString(hResult.value);
  }

  public String toString() {
    return "[SDLIP/CORBA Client Transport]";
  }

  static {
    ClientTransportModule.register("ior", "sdlip.helpers.ClientCorbaTransport");
    ClientTransportModule.register(sdlip.SDLIP.NameServerURISchema, "sdlip.helpers.ClientCorbaTransport");
  }
  
}
