package sdlip.helpers;

import sdlip.DAV;
import sdlip.Search;
import sdlip.ResultAccess;
import sdlip.Metadata;
import sdlip.SDLIP;
import sdlip.SDLIPException;
import sdlip.http.*;
import sdlip.xml.dom.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import org.omg.CORBA.IntHolder;

import java.net.*;
import java.io.*;
import java.util.*;

public class ServerDaslTransport implements HTTPCallback {

  static final int DEFAULT_PORT = 8080;

  //  public static final String INET_ADDRESS = "sdlip.helpers.SDLIPCallback.InetAddress";
  static final String CONTENT_TYPE = "text/xml";

  static final int DEFAULT_SEARCH_TIMEOUT_REQ = 3600;
  static final int DEFAULT_SEARCH_TIMEOUT_REP = 0;
  static final int DEFAULT_SID_REP = 0;
  static final int DEFAULT_NUM_DOCS = sdlip.Search.RETURN_ALL_DOCS; // 10;

  DBG dbg = new DBG();
  HTTPServer httpServer;
  int httpServerPort;

  public ServerDaslTransport(int port) {
    httpServerPort = port;
  }

  public ServerDaslTransport() {
    this(DEFAULT_PORT);
  }

  public ServerDaslTransport(int port, String lspPath, Object lsp) throws IOException {

    this(port);
    addLSP(lspPath, lsp);
  }

  public ServerDaslTransport(int port, Object lsp) throws IOException {

    this(port, "/", lsp);
  }

  public ServerDaslTransport(Object lsp) throws IOException {

    this(DEFAULT_PORT, "/", lsp);
  }

  public void setDBG(DBG dbg) {

    this.dbg = dbg;
  }

  /** lsp path -> Search */
  Hashtable lsps = new Hashtable();

  public void addLSP(Object lsp) throws IOException {
    addLSP("/", lsp);
  }

  public synchronized void addLSP(String lspPath, Object lsp) throws IOException {

    if(!( (lsp instanceof sdlip.Search) ||
	  (lsp instanceof sdlip.ResultAccess)))
      throw new RuntimeException("LSP must at least implement Search or ResultAccess interface");
    if(!lspPath.startsWith("/"))
      lspPath = "/" + lspPath;
    lsps.put(lspPath, lsp);
    if(httpServer == null) {
      dbg.println(DBG.SHORT, "Starting DASL/HTTP server transport on port: " + httpServerPort);
      httpServer = new HTTPServer(this, httpServerPort);
    }
  }

  public Object getLSP(String lspPath) {

    return lsps.get(lspPath);
  }

  String getLSPPath(String line) {

    StringTokenizer st = new StringTokenizer(line);
    st.nextToken();
    return st.nextToken();
  }


  public void process(Socket socket, InputStream in, OutputStream out) throws Exception {


    // SM.util.FileUtils.copyStream(in, System.out);

    PrintWriter pw = new PrintWriter(new OutputStreamWriter(out));

    try {

      dbg.println(DBG.SHORT, toString() + " request from: " + socket.getInetAddress().getHostName());

      BufferedReader din = new BufferedReader(new InputStreamReader( in ));
	  
      String line = din.readLine();
      
      if(line.startsWith("GET ") || line.startsWith("SEARCH ")) {
	
	Object lsp = getLSP(getLSPPath(line));
	if(lsp == null) {
	  dbg.println(DBG.SHORT, toString() + " no LSP for " + getLSPPath(line));
	  return;
	}
	// read header
	Hashtable header = HTTPUtil.getMIMEHeader(din);
	String contentType = (String)header.get("content-type");
	if(contentType == null || !"text/xml".equals(contentType))
	  throw new SDLIPException(SDLIPException.SERVER_ERROR_EXC, "Content type \"text/xml\" expected instead of \"" + contentType + "\"");
	
	int length = 0;
	try {
	  length = Integer.parseInt((String)header.get("content-length"));
	} catch (Exception any) {
	  dbg.println(DBG.SHORT, "No length: " + header.get("content-length")); 
	}
	
	//	System.err.println("== reading " + length);

	char[] buf = new char[length];
	int bufPos = 0, singleRead = 0, totalRead = 0;

	do {
	  singleRead = din.read(buf, bufPos, length - totalRead);
	  totalRead += singleRead;
	} while (totalRead < length && singleRead > 0);
	
	//	din.read(buf, 0, length);

	//	System.err.println("== read " + length + "\n" + new String(buf));
	
	// read xml, invoke search
	XMLObject iRequest = new XMLObject(new String(buf));
	//	InputSource source = new InputSource(new StringReader(new String(buf)));
	//	InputSource source = new InputSource(new ByteArrayInputStream(buf));

	//	Element request = DOMParser.parse(source);
	Element request = iRequest.getElement();


	if(dbg.getLevel() >= DBG.VERBOSE) {
	  if(request != null)
	    dbg.println(DBG.VERBOSE, "--- ORIGINAL REQUEST:\n" + DOMWriter.prettyPrint(request));
	  else
	    dbg.println(DBG.VERBOSE, "--- UNPARSABLE REQUEST:\n" + buf);
	}

	if(request == null)
	  throw new Exception("Empty XML data"); // can happen?

	Element result = null;

	//	ParameterHolder holder = new ParameterHolder();
	//	holder.set(INET_ADDRESS, socket.getInetAddress());

	// FIXME: check whether native DASL client...
	// if so, issue HTTP redirect instead of SDLIP redirect

	// DISPATCHER
	if(DAV.searchrequest.equals(request.getTagName())) {
	  checkSearchInterface(lsp);
	  result = doSearch(request, (sdlip.Search)lsp);

	} else if(SDLIP.GetSessionInfo.equals(request.getTagName())) {
	  checkResultAccessInterface(lsp);
	  result = doGetSessionInfo(request, (sdlip.ResultAccess)lsp);

	} else if(SDLIP.GetDocs.equals(request.getTagName())) {
	  checkResultAccessInterface(lsp);
	  result = doGetDocs(request, (sdlip.ResultAccess)lsp);

	} else if(SDLIP.ExtendStateTimeout.equals(request.getTagName())) {
	  checkResultAccessInterface(lsp);
	  result = doExtendStateTimeout(request, (sdlip.ResultAccess)lsp);

	} else if(SDLIP.CancelRequest.equals(request.getTagName())) {
	  checkResultAccessInterface(lsp);
	  result = doCancelRequest(request, (sdlip.ResultAccess)lsp);
	  /*
	} else if(SDLIP.RemoveDocs.equals(request.getTagName())) {
	  checkResultAccessInterface(lsp);
	  result = doRemoveDocs(request, (sdlip.ResultAccess)lsp);
	  */
	} else if(SDLIP.GetInterface.equals(request.getTagName())) {
	  checkMetadataInterface(lsp);
	  result = doGetInterface(request, (sdlip.Metadata)lsp);

	} else if(SDLIP.GetSubcollectionInfo.equals(request.getTagName())) {
	  checkMetadataInterface(lsp);
	  result = doGetSubcollectionInfo(request, (sdlip.Metadata)lsp);

	} else if(SDLIP.GetPropertyInfo.equals(request.getTagName())) {
	  checkMetadataInterface(lsp);
	  result = doGetPropertyInfo(request, (sdlip.Metadata)lsp);

	} else
	  throw new Exception("Unknown request: " + request.getTagName());

	if(dbg.getLevel() >= DBG.VERBOSE)
	  dbg.println(DBG.VERBOSE, "--- ORIGINAL RESPONSE:\n" + DOMWriter.prettyPrint(result));

	// successful reply header
	pw.println("HTTP/1.1 207 Multi-Status\nContent-Type: text/xml\nContent-Encoding: utf-8");
	pw.println();
	
	if(result != null) {
	  DOMWriter dw = new DOMWriter(pw);
	  dw.print(result);
	}
	pw.flush();

      } else
	throw new SDLIPException(SDLIPException.INVALID_REQUEST_EXC, "Invalid HTTP method: GET or SEARCH expected");

    } catch(SAXException sax) {
      dbg.println(sax);
      error( pw, DAV.eBadRequest, SDLIPException.MALFORMED_XML_EXC, sax.getException() != null ? sax.getException().getMessage() : null );
    } catch (SDLIPException sdlip) {
      dbg.println(sdlip);
      error( pw, DAV.eBadRequest, sdlip.getCode(), sdlip.getMessage());
    } catch (IOException io) {
      dbg.println(io);
      /* do nothing, client closed connection */
    } catch (Exception any) {
      dbg.println(any);
      error( pw, DAV.eBadRequest, SDLIPException.SERVER_ERROR_EXC, "Server exception: " + getTrace(any) /*.getMessage()*/);
    }
    
  }

  String getTrace(Exception any) {
    StringWriter sw = new StringWriter();
    PrintWriter pw = new PrintWriter(sw);
    any.printStackTrace(pw);
    return sw.toString();
  }

  void checkSearchInterface(Object obj) throws SDLIPException {
    if(!(obj instanceof sdlip.Search))
      throw new SDLIPException(SDLIPException.NOT_IMPLEMENTED_EXC, "Search interface not implemented");
  }

  void checkResultAccessInterface(Object obj) throws SDLIPException {
    if(!(obj instanceof sdlip.ResultAccess))
      throw new SDLIPException(SDLIPException.NOT_IMPLEMENTED_EXC, "ResultAccess interface not implemented");
  }

  void checkMetadataInterface(Object obj) throws SDLIPException {
    if(!(obj instanceof sdlip.Metadata))
      throw new SDLIPException(SDLIPException.NOT_IMPLEMENTED_EXC, "Metadata interface not implemented");
  }

  Element doSearch(Element searchrequest, sdlip.Search search) throws Exception {

    Document doc = searchrequest.getOwnerDocument();
	
    /* look for clientSID, stateTimeoutReq, queryOptions and serverDelegate here */
    Element sdlipSpecific = null;

    Element eSubcols = null;
    Element eQueryOptions = null;
    Element ePropList = null;
    Element eQuery = null;
    int numDocs = DEFAULT_NUM_DOCS;
    int clientSID = 0;
    int stateTimeoutReq = DEFAULT_SEARCH_TIMEOUT_REQ;

    Element basicsearch = DOMUtil.getDescendant(searchrequest, DAV.basicsearch);
    if(basicsearch != null) {
      /* this is a DAV:basicsearch request */
      sdlipSpecific = basicsearch;
      eQuery = basicsearch;
      
      /* extract numDocs from DAV:limit */
      try {
	numDocs = Integer.parseInt(DOMUtil.getDescendantText(searchrequest, DAV.nresults));
      } catch (Exception any) {}
      
      /* extract subcols from DAV:from */
      NodeList subcols = basicsearch.getElementsByTagName(DAV.scope);
      if(subcols.getLength() > 0) {
	eSubcols = doc.createElement(SDLIP.subcols);
	for(int i = 0; i < subcols.getLength(); i++) {
	  String uri = DOMUtil.getChildText((Element)subcols.item(i), DAV.href);
	  if(uri != null && uri.startsWith(SDLIP.SID))
	    eSubcols.appendChild( DOMUtil.createTextNode(doc, SDLIP.resSet, uri.substring(SDLIP.SID.length())) );
	  else
	    eSubcols.appendChild( DOMUtil.createTextNode(doc, SDLIP.subcolName, uri) );
	}
      }

      /* extract propList from DAV:select */
      Element select = DOMUtil.getChild(basicsearch, DAV.select);
      Element prop = DOMUtil.getChild(select, DAV.prop);
      if(prop != null) {
	NodeList propList = DOMUtil.getChildElements(prop, "*");
	if(propList.getLength() == 0)
	  throw new SDLIPException(SDLIPException.BAD_QUERY_EXC, "Invalid format of DAV:select: empty property list. Use DAV:allprop");
	ePropList = doc.createElement(SDLIP.propList);
	for(int i=0; i < propList.getLength(); i++) {
	  ePropList.appendChild( propList.item(i).cloneNode(true) );
	}
      } else {
	/* DAV:allprop must be inside DAV:select, do nothing
	 * ePropList remains null, all properties will be delivered */
      }

    } else {
      /* not DAV:basicsearch */
      Element eSearchRequest = DOMUtil.getChild(searchrequest, "*");

      if(eSearchRequest == null)
	throw new SDLIPException(SDLIPException.BAD_QUERY_EXC, "Transport error: no query specified");

      if(SDLIP.SearchRequest.equals(eSearchRequest.getTagName())) {
	/* SDLIP search request */
	sdlipSpecific = eSearchRequest;
	/* get query */
	Element eQW = DOMUtil.getChild(eSearchRequest, SDLIP.query);
	eQuery = DOMUtil.getChild(eQW, "*");
	if(eQuery == null)
	  throw new SDLIPException(SDLIPException.BAD_QUERY_EXC, "No query specified");
	/* extract numDocs from DAV:limit */
	try {
	  numDocs = Integer.parseInt(DOMUtil.getChildText(eSearchRequest, SDLIP.numDocs));
	} catch (Exception any) {}
	/* get propList and subcols, both may be null */
	ePropList = DOMUtil.getChild(eSearchRequest, SDLIP.propList);
	eSubcols = DOMUtil.getChild(eSearchRequest, SDLIP.subcols);
      } else {
	/* custom native DASL query */
	eQuery = eSearchRequest;
      }
    }

    /* extract both clientSID and stateTimeoutReq */

    if(sdlipSpecific != null) {
      /* extract stateTimeoutReq */
      try {
	Element eTR = DOMUtil.getChild(sdlipSpecific, SDLIP.stateTimeoutReq);
	stateTimeoutReq = Integer.parseInt(DOMUtil.getText(eTR));
	sdlipSpecific.removeChild(eTR);
      } catch (Exception any) {}
      /* extract clientSID */
      try {
	Element eCID = DOMUtil.getChild(sdlipSpecific, SDLIP.clientSID);
	clientSID = Integer.parseInt(DOMUtil.getText(eCID));
	sdlipSpecific.removeChild(eCID);
      } catch (Exception any) {}
      /* extract queryOptions */
      Element wrQO = DOMUtil.getChild(sdlipSpecific, SDLIP.queryOptions);
      eQueryOptions = DOMUtil.getChild(wrQO, "*");
      sdlipSpecific.removeChild(wrQO);
    }
	
    IntHolder hExpectedTotal = new IntHolder(sdlip.Search.UNKNOWABLE);
    IntHolder hStateTimeout = new IntHolder(DEFAULT_SEARCH_TIMEOUT_REP);
    IntHolder hServerSID = new IntHolder(DEFAULT_SID_REP);
    XMLObject hServerDelegate = new XMLObject();
    XMLObject hResult = new XMLObject();

    XMLObject EMPTY = new XMLObject();

    // do search
    search.search(clientSID,
		  eSubcols != null ? new XMLObject(eSubcols) : EMPTY,
		  new XMLObject(eQuery),
		  numDocs,
		  ePropList != null ? new XMLObject(ePropList) : EMPTY,
		  stateTimeoutReq,
		  eQueryOptions != null ? new XMLObject(eQueryOptions) : EMPTY,
		  hExpectedTotal,
		  hStateTimeout,
		  hServerSID,
		  hServerDelegate,
		  hResult);

    /* translate the result into DASL */
    Element result = hResult.getElement();

    if(dbg.getLevel() <= DBG.VERBOSE)
      dbg.println(DBG.VERBOSE, "--- ORIGINAL RESPONSE:\n" + hResult.getPrettyString());

    /* create DAV:multistatus reply */
    Element multistatus = doc.createElement(DAV.multistatus);

    /* sdlipResponse is the first DAV:response holding SDLIP-specific parameters */
    Element sdlipResponse = doc.createElement(DAV.response);
    multistatus.appendChild(sdlipResponse);
    sdlipResponse.appendChild( DOMUtil.createTextNode(doc, DAV.href, SDLIP.SearchResult) );
    /* insert expectedTotal, stateTimeout, serverSID and serverDelegate if any */
    sdlipResponse.appendChild( DOMUtil.createTextNode(doc, SDLIP.expectedTotal, String.valueOf(hExpectedTotal.value)) );
    sdlipResponse.appendChild( DOMUtil.createTextNode(doc, SDLIP.stateTimeout, String.valueOf(hStateTimeout.value)) );
    sdlipResponse.appendChild( DOMUtil.createTextNode(doc, SDLIP.serverSID, String.valueOf(hServerSID.value)) );
    Element eServerDelegate = hServerDelegate.getElement();
    if(eServerDelegate != null)
      sdlipResponse.appendChild(eServerDelegate);

    /* create regular responses if any */

    NodeList docList = DOMUtil.getChildElements(result, SDLIP.doc);
    for(int i=0; i < docList.getLength(); i++) {
      /* get next document */
      Element eDoc = (Element)docList.item(i);
      Element eProps = DOMUtil.getChild(eDoc, SDLIP.propList);
      if(eProps == null)
	throw new SDLIPException(SDLIPException.SERVER_ERROR_EXC, "LSP: document with bad or missing " + SDLIP.propList + " element");
      /* create DAV:response and add it to DAV:multistatus */
      Element response = doc.createElement(DAV.response);
      multistatus.appendChild(response);
      /* find DID */
      Element eDID = DOMUtil.getChild(eDoc, SDLIP.DID);
      int DID = 0;
      try {
	DID = Integer.parseInt( DOMUtil.getText(eDID) );
      } catch (Exception any) {}
      if(DID <= 0)
	throw new SDLIPException(SDLIPException.SERVER_ERROR_EXC, "LSP: document with bad or missing DID");
      /* create a fake DAV:href */
      response.appendChild( DOMUtil.createTextNode(doc, DAV.href, SDLIP.fakeDID + DID) );
      /* move other elements (do not copy, may contains big binary data!)
       * getChildElements is stable, so we can do it... */
      Element propstat = doc.createElement(DAV.propstat);
      Element prop = doc.createElement(DAV.prop);
      propstat.appendChild(prop);
      response.appendChild(propstat);
      NodeList propList = DOMUtil.getChildElements(eProps, "*");
      for(int j=0; j < propList.getLength(); j++) {
	Element p = (Element)propList.item(j);
	if(eDID != p)
	  prop.appendChild(p);
      }
      /* append status HTTP/1.1 200 */
      propstat.appendChild( DOMUtil.createTextNode(doc, DAV.status, "HTTP/1.1 200 OK") );
      response.appendChild( eDID );
    }   
    return multistatus;
  }

  Element doCancelRequest(Element request, ResultAccess resultAccess) throws Exception {

    /* extract serverSID */
    int serverSID = 0;
    try {
      serverSID = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.serverSID));
    } catch (Exception any) {}
    
    /* extract reqID */
    int reqID = 0;
    try {
      reqID = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.reqID));
    } catch (Exception any) {}

    resultAccess.cancelRequest(serverSID, reqID);
    return null;
  }

  Element doGetSessionInfo(Element request, ResultAccess resultAccess) throws Exception {

    /* extract serverSID */
    int serverSID = 0;
    try {
      serverSID = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.serverSID));
    } catch (Exception any) {}
    
    IntHolder hExpectedTotal = new IntHolder();
    IntHolder hStateTimeout = new IntHolder();

    /* invoke original method */
    resultAccess.getSessionInfo(serverSID, hExpectedTotal, hStateTimeout);

    /* package result */
    Document doc = XMLObject.getDocument();
    Element result = doc.createElement(SDLIP.SessionInfo);
    result.appendChild( DOMUtil.createTextNode(doc, SDLIP.expectedTotal, String.valueOf(hExpectedTotal.value)) );
    result.appendChild( DOMUtil.createTextNode(doc, SDLIP.stateTimeout, String.valueOf(hStateTimeout.value)) );

    return result;
  }

  Element doGetDocs(Element request, ResultAccess resultAccess) throws Exception {

    /* extract serverSID */
    int serverSID = 0;
    try {
      serverSID = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.serverSID));
    } catch (Exception any) {}
    
    /* extract reqID */
    int reqID = 0;
    try {
      reqID = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.reqID));
    } catch (Exception any) {}

    /* extract propList, may be null */
    Element ePropList = DOMUtil.getChild(request, SDLIP.propList);

    /* extract docSelection */
    String docSelection = DOMUtil.getChildText(request, SDLIP.docSelection);
    if(docSelection == null)
      throw new SDLIPException(SDLIPException.INVALID_REQUEST_EXC, "docSelection element is missing");

    XMLObject res = new XMLObject();

    /* invoke original method */
    resultAccess.getDocs(serverSID, reqID, new XMLObject(ePropList), docSelection, res);

    return res.getElement();
  }


  Element doExtendStateTimeout(Element request, ResultAccess resultAccess) throws Exception {

    /* extract serverSID */
    int serverSID = 0;
    try {
      serverSID = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.serverSID));
    } catch (Exception any) {}
    
    /* extract stateTimeoutReq */
    int stateTimeoutReq = 0;
    try {
      stateTimeoutReq = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.stateTimeoutReq));
    } catch (Exception any) {}
    
    IntHolder hStateTimeout = new IntHolder();

    /* invoke original method */
    resultAccess.extendStateTimeout(serverSID, stateTimeoutReq, hStateTimeout);

    /* package result */
    Document doc = XMLObject.getDocument();
    Element result = doc.createElement(SDLIP.SessionInfo);
    result.appendChild( DOMUtil.createTextNode(doc, SDLIP.stateTimeout, String.valueOf(hStateTimeout.value)) );

    return result;
  }

  /*
  Element doRemoveDocs(Element request, ResultAccess resultAccess) throws Exception {

    /* extract serverSID * /
    int serverSID = 0;
    try {
      serverSID = Integer.parseInt(DOMUtil.getChildText(request, SDLIP.serverSID));
    } catch (Exception any) {}
    
    /* extract docSelection * /
    String docSelection = DOMUtil.getChildText(request, SDLIP.docSelection);
    if(docSelection == null)
      throw new SDLIPException(SDLIPException.INVALID_REQUEST_EXC, "docSelection element is missing");

    /* invoke original method * /
    resultAccess.removeDocs(serverSID, docSelection);

    /* no result * /
    return null;
  }
  */

  Element doGetInterface(Element request, Metadata metadata) throws Exception {
    
    XMLObject res = new XMLObject();
    /* invoke original method */
    metadata.getInterface(res);
    
    /* deliver result */
    return res.getElement();
  }

  Element doGetSubcollectionInfo(Element request, Metadata metadata) throws Exception {
    
    XMLObject res = new XMLObject();
    /* invoke original method */
    metadata.getSubcollectionInfo(res);
    
    /* deliver result */
    return res.getElement();
  }

  Element doGetPropertyInfo(Element request, Metadata metadata) throws Exception {
    
    /* extract subcolName */
    String subcolName = DOMUtil.getChildText(request, SDLIP.subcolName);
    if(subcolName == null)
      throw new SDLIPException(SDLIPException.INVALID_REQUEST_EXC, "subcolName element is missing");

    XMLObject res = new XMLObject();
    /* invoke original method */
    metadata.getPropertyInfo(subcolName, res);
    
    /* deliver result */
    return res.getElement();
  }

  void error(PrintWriter pw, int DAVErrorCode, int SDLIPErrorCode, String desc) {

    pw.println("HTTP/1.1 " + DAVErrorCode);
    pw.println(CONTENT_TYPE);
    pw.println();
    Document doc = new DocumentImpl();
    Element multistatus = doc.createElement(DAV.multistatus);
    Element response = doc.createElement(DAV.response);
    Element href = DOMUtil.createTextNode(doc, DAV.href, SDLIP.exception);
    multistatus.appendChild(response);
    response.appendChild(href);
    Element status = DOMUtil.createTextNode(doc, SDLIP.exceptionCode, String.valueOf(SDLIPErrorCode));
    response.appendChild(status);
    if(desc != null) {
      Element eDesc = DOMUtil.createTextNode(doc, DAV.responsedescription, desc);
      response.appendChild(eDesc);
    }
    DOMWriter dw = new DOMWriter(pw);
    dw.setAutoIndent(true); // FIXME?
    dw.print(multistatus);
    pw.flush();
  }

  public String toString() {
    return "[SDLIP/DASL Server Transport]";
  }
}
