package sdlip.xml.dom;

import sdlip.SDLIP;

import org.w3c.dom.*;
import org.w3c.tools.codec.*;
import java.io.*;

/**
 * This is a DOM toolkit described in the SDLIP Java tutorial.
 */

public class DOMUtil {

  public static void base64Decode(Element bin, OutputStream out) throws IOException, Base64FormatException {
    
    if(!SDLIP.base64.equals(bin.getTagName()))
      throw new Base64FormatException("Invalid tag name: " + bin.getTagName() + ". Expected: " + SDLIP.base64);
    
    String input = bin.getFirstChild().getNodeValue();
    byte bytes[] = new byte[input.length()] ;
    input.getBytes (0, bytes.length, bytes, 0) ;
    
    Base64Decoder decoder = new Base64Decoder( new ByteArrayInputStream(bytes), out );
    decoder.process();
  }

  public static byte[] base64Decode(Element bin) throws IOException, Base64FormatException {

    ByteArrayOutputStream out = new ByteArrayOutputStream();
    base64Decode(bin, out);
    return out.toByteArray();
  }

  public static Element base64Encode(Document doc, InputStream in) throws IOException {
    
    ByteArrayOutputStream out = new ByteArrayOutputStream () ;
    Base64Encoder encoder = new Base64Encoder(in, out);
    encoder.process();
    return DOMUtil.createTextNode(doc, SDLIP.base64, out.toString());
  }

  public static Element base64Encode(Document doc, byte[] bytes) throws IOException {

    return base64Encode(doc, new ByteArrayInputStream(bytes));
  }

  public static Document createDocument() {
    return new DocumentImpl();
  }

  public static Element createTextNode(Document doc, String name, String value) {

    Element node = doc.createElement(name);
    Node text = doc.createTextNode(value);
    node.appendChild(text);
    return node;
  }

  public static Element getChild(Element elem, String name) {

    if(elem == null)
      return null;
    Node p = elem.getFirstChild();
    while(p != null) {
      if(p.getNodeType() == Node.ELEMENT_NODE &&
	 ("*".equals(name) || p.getNodeName().equals(name)))
	return (Element)p;
      p = p.getNextSibling();
    }
    return null;
  }

  /**
   * use "*" as name to get all child elements
   */
  public static NodeList getChildElements(Element elem, String name) {

    NodeListImpl list = new NodeListImpl();
    Node p = elem.getFirstChild();
    while(p != null) {
      if(p.getNodeType() == Node.ELEMENT_NODE &&
	 ("*".equals(name) || p.getNodeName().equals(name)))
	list.addElement(p);
      p = p.getNextSibling();
    }
    return list;
  }

  public static String getChildText(Element elem, String name) {

    Element el = getChild(elem, name);
    return getText(el);
  }

  public static Element getDescendant(Element elem, String name) {
    
    return (Element)elem.getElementsByTagName(name).item(0);
  }

  public static String getDescendantText(Element elem, String name) {
    
    Element el = (Element)elem.getElementsByTagName(name).item(0);
    return getText(el);
  }

  public static String getText(Element el) {
    if(el != null) {
      Node txt = el.getFirstChild();
      if(txt != null)
	return txt.getNodeValue();
    }
    return null;
  }

}
