package sdlip.xml.dom;

import org.w3c.dom.*;
import java.util.Vector;

public class DeepNodeListImpl 
    implements NodeList {

    // Data

    protected NodeImpl rootNode;      // Where the search started
    protected String tagName;               // Or "*" to mean all-tags-acceptable
    protected int changes=0;
    protected Vector nodes;

    // Constructors

    /** Constructor. */
    protected DeepNodeListImpl(NodeImpl rootNode, String tagName) {
	    this.rootNode = rootNode;
	    this.tagName  = tagName;
	    nodes = new Vector();
        }  

    // NodeList methods

    /** Returns the length of the node list. */
    public int getLength() {
	    // Preload all matching elements. (Stops when we run out of subtree!)
	    item(java.lang.Integer.MAX_VALUE);
	    return nodes.size();
    }  

    /** Returns the node at the specified index. */
    public Node item(int index) {
    	Node thisNode;

        // In the cache
    	if (index < nodes.size())      
    	    return (Node)nodes.elementAt(index);
    
        // Not yet seen
    	else {
    
            // Pick up where we left off (Which may be the beginning)
    		if (nodes.size() == 0)     
    		    thisNode = rootNode;
    		else
    		    thisNode=(NodeImpl)(nodes.lastElement());
    
    		// Add nodes up to the one we're looking for
    		while(thisNode != null && index >= nodes.size()) {
    			thisNode=nextMatchingElementAfter(thisNode);
    			if (thisNode != null)
    			    nodes.addElement(thisNode);
    		    }

            // Either what we want, or null (not avail.)
		    return thisNode;           
	    }

    } // item(int):Node

    // Private methods

    private Node nextMatchingElementAfter(Node current) {

      Node next;
      while (current != null) {
	// Look down to first child.
	if (current.hasChildNodes()) {
	  current = (current.getFirstChild());
	}

	// Look right to sibling (but not from root!)
	else if (current != rootNode && null != (next = current.getNextSibling())) {
	  current = next;
	}

	// Look up and right (but not past root!)
	else {
	  next = null;
	  for (; current != rootNode; // Stop when we return to starting point
	       current = current.getParentNode()) {
	    
	    next = current.getNextSibling();
	    if (next != null)
	      break;
	  }
	  current = next;
	}
	
	// Have we found an Element with the right tagName?
	// ("*" matches anything.)
	//	if(current != null)
	//	  System.err.println("--- CHECKING " + current.getNodeName() + " against " + tagName);
	
	if (current != rootNode 
	    && current != null
	    && current.getNodeType() ==  Node.ELEMENT_NODE 
	    && (tagName.equals("*") 
		|| ((ElementImpl) current).getTagName().equals(tagName))
	     ) 
	  {
	    return current;
	  }
	
	// Otherwise continue walking the tree
      }
      
      // Fell out of tree-walk; no more instances found
      return null;
      
    } // nextMatchingElementAfter(int):Node

} // class DeepNodeListImpl
