package sdlip.xml.dom;

import org.w3c.dom.*;

import java.util.*;

public class DocumentImpl extends NodeImpl implements Document {

  public DocumentImpl() {
    super(null, null, null);
    ownerDocument = this;
  }

  public DocumentType       getDoctype() {
    return null;
  }

  public DOMImplementation  getImplementation() {
    return null;
  }

  public Element            getDocumentElement() { 

    return (Element)getFirstChild();
  }

  protected void        setDocumentElement(Element root) { 

    if(getFirstChild() != null)
      removeChild(getFirstChild());
    appendChild(root);
  }

  public Element            createElement(String tagName)
    throws DOMException {

    return new ElementImpl(this, tagName);
  }

    /** Returns the node type. */
    public short getNodeType() {
        return Node.DOCUMENT_NODE;
    }

    /** Returns the node name. */
    public String getNodeName() {
        return "#document";
    }

  public DocumentFragment   createDocumentFragment() {
    throw new DOMExceptionImpl(DOMException.NOT_SUPPORTED_ERR, null);
  }

  public Text               createTextNode(String data) {
    return new TextImpl(this, data);
  }

  public Comment            createComment(String data) {
    throw new DOMExceptionImpl(DOMException.NOT_SUPPORTED_ERR, null);
  }

  public CDATASection       createCDATASection(String data)
    throws DOMException {

    throw new DOMExceptionImpl(DOMException.NOT_SUPPORTED_ERR, null);
  }

  public ProcessingInstruction createProcessingInstruction(String target, 
                                                           String data)
    throws DOMException {

    throw new DOMExceptionImpl(DOMException.NOT_SUPPORTED_ERR, null);
  }

  public Attr               createAttribute(String name)
    throws DOMException {

    return new AttrImpl(this, name);
  }

  public EntityReference    createEntityReference(String name)
    throws DOMException {

    throw new DOMExceptionImpl(DOMException.NOT_SUPPORTED_ERR, null);
  }

  public NodeList getElementsByTagName(String tagname) {
    return new DeepNodeListImpl(this,tagname);
  }  

  Hashtable namespaces = new Hashtable();
  int nscounter = 0;

  public void registerNamespace(String ns, String abb) {
    
    if(ns == null || ns.length() == 0 || namespaces.containsKey(ns))
      return;

    if(abb == null || abb.length() == 0 || namespaces.contains(abb))
      namespaces.put(ns, String.valueOf((char)((int)'a' + nscounter++)));
    else
      namespaces.put(ns, abb);
  }

  public void registerNamespace(String ns) {
    
    //    System.err.println("---NEW NS: " + ns);
    registerNamespace(ns, null);
  }

  protected Hashtable getNamespaces(Element element) {
    
    Hashtable t = new Hashtable();
    getNamespaces(t, element);
    return t;
  }

  void getNamespaces(Hashtable t, Element el) {
    
    extractNS(t, el.getTagName());
    NodeList list = el.getElementsByTagName("*");
    for(int i = 0; i < list.getLength(); i++) {

      Element e = (Element)list.item(i);
      extractNS(t, e.getTagName());

      NamedNodeMap map = e.getAttributes();
      for(int j = 0; j < map.getLength(); j++) {
	Attr att = (Attr)map.item(j);
	extractNS(t, att.getName());
      }
    }
  }

  int getNSEnd(String name) {
    // first try to detemine namespace directly
    int l = name.length()-1;
    do {
      char c = name.charAt(l);
      if(c == '#' || c == ':' || c == '/')
	break;
      l--;
    } while (l >= 0);
    l++;
    return l;
  }

  void extractNS(Hashtable t, String name) {
    
    int l = getNSEnd(name);
    if(l > 0) {
      String ns = name.substring(0, l);
      if(!t.containsKey(ns)) {
	//	System.err.println("---PUT : " + ns);
	t.put(ns, getShortcut(t.size()));
      }
    }
  }

  String getShortcut(int counter) {
    return counter < (int)'s' ? String.valueOf((char)((int)'a' + counter)) : "n" + (counter - (int)'s' + 1);
  }

  public String getShort(Hashtable namespaces, String name) {

    int l = getNSEnd(name);

    String ns = null;
    if(l > 0) {
      
      ns = name.substring(0, l);
      String abb = (String)namespaces.get(ns);
      if(abb != null)
	return abb + ":" + name.substring(l);
    }
    
    // no delimiter in the namespace, ambiguities possible :(
    
    for(Enumeration en = namespaces.keys(); en.hasMoreElements(); ) {
      String key = (String)en.nextElement();
      if(name.startsWith(key))
	return (String)namespaces.get(key) + ":" + name.substring(key.length());
    }

    if(ns != null) {
      throw new RuntimeException("DocumentImpl: unknown namespace of " + name);
      // register new namespace
      //      registerNamespace(ns);
      //      return (String)namespaces.get(ns) + ":" + name.substring(l);
    } else
      // unqualified
      return name;
  }

  public String toString() {

    return "[#document: " + Integer.toHexString(hashCode()) + "]";
  }
}

