package sdlip.xml.dom;

import org.w3c.dom.*;
import java.util.Vector;
import java.util.Enumeration;

public class NamedNodeMapImpl 
    implements NamedNodeMap {

    //
    // Data
    //

    /** Nodes. */
	protected Vector nodes = new Vector();

    /** Owner document. */
	protected Document ownerDoc; // support for WRONG_DOCUMENT_ERR

    /** Default nodes. */
	protected NamedNodeMapImpl defaults;

    /** Element node index for fast named node map. */
    protected int elementNode = -1;

    //
    // Constructors
    //

    /** Constructs a named node map. */
    protected NamedNodeMapImpl(Document ownerDoc) {
	    this.ownerDoc=ownerDoc;
    }

    //
    // NamedNodeMap methods
    //

    public int getLength() {
    	return (nodes != null) ? nodes.size() : 0;
    }
    
    public Node item(int index) {
    	return (nodes != null && index < nodes.size()) ? 
                    (Node)(nodes.elementAt(index)) : null;
    }

    public Node getNamedItem(String name) {

    	int i = findNamePoint(name);
        return (i < 0) ? null : (Node)(nodes.elementAt(i));

    } // getNamedItem(String):Node

    public Node setNamedItem(Node arg) 
        throws DOMException {

    	if(arg.getOwnerDocument()!=ownerDoc) {
            throw new DOMExceptionImpl(DOMException.WRONG_DOCUMENT_ERR,null);
        }
    
    	int i=findNamePoint(arg.getNodeName());
    	Node previous=null;
    	if (i >= 0) {
    		previous=(Node)nodes.elementAt(i);
    		nodes.setElementAt(arg,i);
    	}
    	else {
    		i = -1 - i; // Insert point (may be end of list)
    		if (null == nodes) {
    			nodes = new Vector();
            }
    		nodes.insertElementAt(arg,i);
        }			

    	return previous;

    } // setNamedItem(Node):Node
    
    public Node removeNamedItem(String name) 
        throws DOMException {
    
    	int i = findNamePoint(name);
    	if (i < 0) {
    		throw new DOMExceptionImpl(DOMException.NOT_FOUND_ERR, null);
        }
    
        Node n = (Node)(nodes.elementAt(i));
        // If there's a default, add it instead
        Node d;
        if (defaults != null && 
            (d = defaults.getNamedItem(name)) != null) {
            nodes.setElementAt(d, i);
        }
        else {
            nodes.removeElementAt(i);
        }
            
        return n;
    
    } // removeNamedItem(String):Node
        
    //
    // Public methods
    //

    public NamedNodeMapImpl cloneMap() {

    	boolean deep = true;
    	NamedNodeMapImpl newmap = new NamedNodeMapImpl(ownerDoc);
    	if (nodes != null) {
    		newmap.nodes = new Vector(nodes.size());
    		for (int i = 0; i < nodes.size(); ++i) {
    			newmap.nodes.addElement(((NodeImpl) nodes.elementAt(i)).cloneNode(deep));
            }
        }
    	newmap.defaults = defaults;
    	return newmap;

    } // cloneMap():NamedNodeMapImpl

    //
    // Private methods
    //

    private int findNamePoint(String name) {

    	// Binary search
    	int i = 0;
    	if(nodes != null) {
    		int first = 0;
            int last  = nodes.size() - 1;
    
    		while (first <= last) {
    			i = (first + last) / 2;
    			int test = name.compareTo(((Node)(nodes.elementAt(i))).getNodeName());
    			if(test == 0) {
    				return i; // Name found
                }
    			else if (test < 0) {
    				last = i - 1;
                }
    			else {
    				first = i + 1;
                }
    		}

    		if (first > i) {
                i = first;
            }
    	}

    	return -1 - i; // not-found has to be encoded.

    } // findNamePoint(String):int


} // class NamedNodeMapImpl
